/*
  Java gfx-test framework

   Copyright (C) 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.BasicStroke;
import java.awt.Graphics2D;



import org.gfxtest.callbacks.CommonArcDrawCallbacks;
import org.gfxtest.framework.ColorPalette;
import org.gfxtest.framework.GrayscalePalette;
import org.gfxtest.framework.HSBPalette;
import org.gfxtest.framework.PrintTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test suite checks rendering of various arcs.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.PRINT_TEST)
@GraphicsPrimitive(GraphicsPrimitives.ARC)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class PrintTestArcs extends PrintTest
{

    /**
     * Method which renders set of arcs using various colors and
     * stroke styles. For each arc, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param radiusStep
     *            between two near arcs
     * @param arcDrawCallback
     *            class containing set of callback methods
     */
    private void drawArcs(TestImage image, Graphics2D graphics2d, int radiusStep, CommonArcDrawCallbacks arcDrawCallback)
    {
        // setup rendering
        arcDrawCallback.setup(image, graphics2d);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of arc center
        final int xc = width >> 1;
        final int yc = height >> 1;

        // maximum radius
        final int maxRadius = Math.min(width, height) - BORDER;

        // index to color palette
        int colorIndex = 0;

        // arc start angle
        int startAngle = 0;

        // draw all arcs onto a paper
        for (int radius = MINIMUM_RADIUS; radius < maxRadius; radius += radiusStep)
        {
            // setup can be made for each arc
            arcDrawCallback.iterationCallBack(xc, yc, radius, maxRadius, colorIndex++);
            startAngle += 4;
            // render the arc
            graphics2d.drawArc(xc - radius, yc - radius, radius << 1, radius << 1, startAngle, 270);
        }

        // cleanup rendering
        arcDrawCallback.cleanup();
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcBasicStyle(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // nothing need to be changed
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcColorPalette(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set arc color
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcColorPaletteInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set arc color
                this.graphics.setColor(ColorPalette.getColor(MAX_COLOR_INDEX - index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a hue palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcColorScale(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute hue value
                float hue = radius * 1.0f / maxRadius;
                // set arc color
                this.graphics.setColor(HSBPalette.createHsbColor(hue));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a hue palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcColorScaleInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute hue value
                float hue = 1.0f - radius * 1.0f / maxRadius;
                // set arc color
                this.graphics.setColor(HSBPalette.createHsbColor(hue));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a grayscale palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcGrayScale(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute grayscale value
                float gray = radius * 1.0f / maxRadius;
                // clamp the grayscale value
                if (gray > 1.0f)
                {
                    gray = 1.0f;
                }
                // set arc color
                this.graphics.setColor(GrayscalePalette.createGrayscaleColor(gray));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a grayscale palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcGrayScaleInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // compute grayscale value
                float gray = 1.0f - radius * 1.0f / maxRadius;
                // clamp the grayscale value
                if (gray < 0.0f)
                {
                    gray = 0.0f;
                }
                // set arc color
                this.graphics.setColor(GrayscalePalette.createGrayscaleColor(gray));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and default end caps.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidth(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set new stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapButtJoinBevel(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapRoundJoinBevel(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapSquareJoinBevel(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to miter style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapButtJoinMiter(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to miter style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapRoundJoinMiter(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to miter style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapSquareJoinMiter(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapButtJoinRound(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapRoundJoinRound(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapSquareJoinRound(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MIN_STROKE_WIDTH;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // increase stroke width
                this.strokeWidth  = this.strokeWidth < MAX_STROKE_WIDTH ? this.strokeWidth + STROKE_WIDTH_DELTA : this.strokeWidth;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and default end caps.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set new stroke width
                this.graphics.setStroke(new BasicStroke(this.strokeWidth));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapButtJoinBevelInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapRoundJoinBevelInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapSquareJoinBevelInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to miter style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapButtJoinMiterInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to miter style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapRoundJoinMiterInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to miter style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapSquareJoinMiterInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_BUTT.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapButtJoinRoundInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_ROUND.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapRoundJoinRoundInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arcs are rendered with various width and end caps set to CAP_SQUARE.
     * Join style is set to bevel style.
     * Color of all rendered arcs are set to black.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcChangeWidthCapSquareJoinRoundInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP * 3 / 2, new CommonArcDrawCallbacks()
        {
            /**
             * Stroke width which is changed in each iteration.
             */
            private float strokeWidth = MAX_STROKE_WIDTH - 2;

            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set stroke width and style
                this.graphics.setStroke(new BasicStroke(this.strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND));
                // change stroke width for the next iteration
                changeStrokeWidth();
                return;
            }

            /**
             * Changes stroke width.
             */
            private void changeStrokeWidth()
            {
                // decrease stroke width
                this.strokeWidth -= STROKE_WIDTH_DELTA;
                // stroke width should not be less than zero
                if (this.strokeWidth < MIN_STROKE_WIDTH)
                {
                    this.strokeWidth = MIN_STROKE_WIDTH;
                }
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            not used in this case
     */
    public static void main(String[] args)
    {
        new PrintTestArcs().runTestSuite(args);
    }
}
